from sqlalchemy import Column, Integer, Unicode, String
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, relationship
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.schema import ForeignKey
from cmpypass.helpers.log_helpers import LogLevels, log
import os

from cmpypass.helpers.route_helpers import db_file_route


engine = create_engine(f'sqlite:///{db_file_route}')
Base = declarative_base(bind=engine)


class User(Base):
    __tablename__ = 'user'
    __table_args__ = {'extend_existing': True}
    id = Column(Integer, primary_key=True)
    name = Column(Unicode(40))
    password = Column(String(200))

    def __init__(self, name, password=None):
        self.name = name
        self.password = password


class Pypass(Base):
    __tablename__ = 'pypass'
    __table_args__ = {'extend_existing': True}
    id = Column(Integer, primary_key=True)
    user_id = Column(Integer, ForeignKey(User.id))
    parent = relationship(User, cascade="all,delete")
    name = Column(Unicode(40), nullable=False)
    username = Column(Unicode(40), nullable=False)
    category = Column(String(20), nullable=False)
    password = Column(String(200), nullable=False)

    def __init__(self, name, username, password, category, user_id):
        self.username = username
        self.name = name
        self.password = password
        self.user_id = user_id
        self.category = category


class Category(Base):
    __tablename__ = 'category'
    __table_args__ = {'extend_existing': True}
    id = Column(Integer, primary_key=True)
    name = Column(Unicode(40))
    user_id = Column(Integer, ForeignKey(User.id))
    parent = relationship(User, cascade="all,delete")

    def __init__(self, name, user_id):
        self.name = name
        self.user_id = user_id


def create_db():
    Base.metadata.create_all()
    log(LogLevels.DEBUG, f"Created SQLite tables at location {db_file_route}")

def remove_db():
    try:
        os.remove(db_file_route)
        log(LogLevels.DEBUG, f"Removed database file at {db_file_route}")
    except Exception as e:
        log(LogLevels.ERROR, str(e))


def get_session():
    Session = sessionmaker(bind=engine)
    return Session()
