
from sqlalchemy.orm.session import Session
from termcolor import colored
from tabulate import tabulate

from cmpypass.db import Pypass, get_session, Category
from cmpypass.services.cache_svc import get_logged_user

logged_user = get_logged_user()

def category_exists(name, id=None):
    user_id = id if id is not None else logged_user.id
    s:Session = get_session()
    category = s.query(Category).filter(Category.name == name, (Category.user_id == user_id)).first()
    s.close()
    if category is None:
        return False
    return True

def add_category(name, id=None):
    user_id = id if id is not None else logged_user.id
    s:Session = get_session()
    if not category_exists(name, user_id):
        c = Category(name, user_id)
        s.add(c)
        s.commit()
        print(f'Category {colored(name, "magenta")} {colored("succesfully created", "green")}')
    else:
        print(colored(f'Category {name} already exists', "red"))
    s.close()

def delete_category(name, id=None):
    user_id = id if id is not None else logged_user.id
    s:Session = get_session()
    if category_exists(name, user_id):
        print(colored(" This is a potentially destructive action, all the passwords with this category will be removed, are you sure (y/n)? ", "yellow"))
        response = input("> ") if id is None else "y"
        if response == "y":
            s.query(Category).filter(Category.name == name, (Category.user_id == user_id)).delete()
            s.query(Pypass).filter(Pypass.category == name, (Pypass.user_id == user_id)).delete()
            s.commit()
            print(f'Category {colored(name, "magenta")} {colored("succesfully deleted", "green")}')
            return
    else:
        print(f'Category {colored(name, "magenta")} {colored("does not exist", "red")}')
    s.close()

def list_categories():
    s:Session = get_session()
    categories = s.query(Category).filter(Category.user_id == logged_user.id)
    s.close()
    table_list = []
    for category in categories:
        row = [category.id, category.name, s.query(Pypass).filter(Pypass.category == category.name and Pypass.user_id == logged_user.id).count()]
        table_list.append(row)
    print(tabulate(table_list, headers=["id", "name", "count"], tablefmt="github"))