import unittest

import pyperclip
from cmpypass.db import create_db, get_session, remove_db
from cmpypass.services.category_svc import *
from cmpypass.services.pypass_svc import *
from cmpypass.services.user_svc import *


class TestServices(unittest.TestCase):
    def setUp(self):
        create_db()

    def tearDown(self):
        remove_db()

    def test_category(self):
        print(colored("\n===============================", "cyan"))
        print(colored(">>> Testing category service", "cyan"))
        print(colored("===============================\n", "cyan"))

        s: Session = get_session()
        create_user("carlos", "1234")
        user: User = s.query(User).filter(User.name == "carlos").first()

        add_category("test", user.id)
        category = s.query(Category).filter(
            Category.name == "test", (Category.user_id == user.id)).first()
        self.assertIsNotNone(category)

        delete_category("test", user.id)
        category = s.query(Category).filter(
            Category.name == "test", (Category.user_id == user.id)).first()
        self.assertIsNone(category)

        s.close()

    def test_pypass(self):
        print(colored("\n===============================", "cyan"))
        print(colored(">>> Testing pypass service", "cyan"))
        print(colored("===============================\n", "cyan"))

        s: Session = get_session()
        create_user("carlos", "1234")
        user: User = s.query(User).filter(User.name == "carlos").first()

        add_category("testcategory", user.id)
        add_pypass("test", "testusername", "testcategory",
                   "testpassword", user.id)

        pypass = s.query(Pypass).filter(Pypass.name == "test", (Pypass.category ==
                                                                "testcategory"), (Pypass.user_id == user.id)).first()
        self.assertIsNotNone(pypass)

        copy_pypass("test", "uname", user.id)
        self.assertEqual(pyperclip.paste(), "testusername")
        copy_pypass("test", "pwd", user.id)
        self.assertEqual(pyperclip.paste(), "testpassword")

        delete_pypass("test", user.id)
        pypass = s.query(Pypass).filter(Pypass.name == "test", (Pypass.category ==
                                                                "testcategory"), (Pypass.user_id == user.id)).first()
        self.assertIsNone(pypass)
        
        s.close()

    def test_encryption(self):
        print(colored("\n===============================", "cyan"))
        print(colored(">>> Testing encryption service", "cyan"))
        print(colored("===============================\n", "cyan")) 
        
        encrypted = get_encrypted_password("test")
        print(f">>> Encrypted: {encrypted}")
        decrypted = get_decrypted_password(encrypted)
        print(f">>> Decrypted: {decrypted}")
        self.assertEqual("test", decrypted)


if __name__ == '__main__':
    unittest.main()
